#!/bin/bash

x11vnc_cmd=x11vnc			# or use full path (or set PATH).
pw_file=/etc/pointdev/.vnc_passwd	# password file
log_file=/var/log/x11vnc.log		# log file
conf_file=/etc/pointdev/conf_x11vnc	# config file
display=:0				# display of interest
restart_sleep=1				# pause between X server restarts.
x11vnc_args="-tightfilexfer -forever -nap -o $log_file -display $display -rfbauth $pw_file"

get_xauthority_file()
{
    for i in 1 2 3; do
	patt=" \-auth "
	xauth=$(ps wwwaux | grep "$patt" \
	    | egrep -v 'grep|Xprt' | head -1 \
	    | sed -e 's/^.*-auth//' | awk '{print $1}')
	if [ -n "$xauth" ]; then
	    break
	fi
	sleep 2
    done
    echo $xauth
}

check_conf_process()
{
    PID=$(pidof -o %PPID x11vnc)
    prev_date=$(date -r $conf_file)
    if [ -n "$PID" ]; then
	$x11vnc_cmd -remote "script:$conf_args"
    fi
    while [ -n "$PID" ]; do
	date=$(date -r $conf_file)
	if [ "$prev_date" != "$date" ]; then
	    echo "`date`  $conf_file Changed"
	    . $conf_file
	    $x11vnc_cmd -remote "script:rfbport:$port;$conf_args"
	fi
	sleep $restart_sleep
	prev_date="$date"
	PID=$(pidof -o %PPID x11vnc)
    done
}

stop()
{
    PID=$(pidof -o %PPID -x x11vnc.sh)
    [ -n "$PID" ]  && kill $PID &> /dev/null
    if [ "$?" -gt "0" ]; then
	echo "`date`  stop x11vnc: Failed"
    else
	echo "`date`  stop x11vnc: Success"
	PID=$(pidof -o %PPID x11vnc)
	[ -n "$PID" ]  && kill $PID &> /dev/null
    fi
}

restart()
{
    PID=$(pidof -o %PPID x11vnc)
    [ -n "$PID" ] && kill $PID &> /dev/null
    if [ "$?" -gt "0" ]; then
	echo "`date`  restart x11vnc: Failed"
    else
	echo "`date`  restart x11vnc: Success"
    fi
}

start()
{
    try=1
    while [ 1 ]; do
	echo "`date`  $0 try number: $try"
	try=$(expr "$try" + 1)
	auth=$(get_xauthority_file)
	. $conf_file
	if [ ! -r "$auth" ]; then
	    echo "`date`  bad auth file: \"$auth\""
	else
	    cmd="$x11vnc_cmd $x11vnc_args -auth $auth -rfbport $port"
	    echo "`date`  running: $cmd"
	    $cmd &
	    sleep 1
	    check_conf_process
	fi
	sleep $restart_sleep
    done
}

case $1 in
    start)
	PID=$(pidof -o %PPID -x x11vnc.sh)
	if [ -n "$PID" ]; then
	    echo "`date`  x11vnc already running pid=$PID"
	else
	    start
	fi
	;;
    stop)
	stop
	;;
    restart)
	restart
	;;
    *)
	echo "usage: $0 {start|stop|restart}"
	;;
esac
